<?php

namespace Scriptor\Modules\SimpleComponents;

use Imanager\Category;
use Imanager\Field;
use Imanager\Item;
use Imanager\TemplateParser;
use Imanager\Util;
use Scriptor\Core\Helper;
use Scriptor\Core\Module;
use Scriptor\Core\Scriptor;

class ComponentsEditor extends Module
{
    
    /**
     * @var object - Imanager\Components category 
     */
    private $components; 

    /**
     * @var object - Imanager\Item category
     */
    private $component;
    
    /**
     * @var object - Imanager\TemplateParser instance
     */
    private $tplParser;

    /**
     * @var array - An array of templates 
     */
    private $tpls = [];

	/**
	 * @var boolean - The purpose of this variable is likely to keep track of 
	 * whether the class has been initialized or not. Used to ensure that a 
	 * class is only initialized once, even if it is instantiated multiple times.
	 */
	private static $initialized = false;

    /**
     * This method is called after __construct().
     * It is used to set internal variables and 
     * to enter configuration data into the module.
     * 
     * NOTE: Remember to always call the 
     *       "parent::init()" method!
     * 
     * @return void
     */
    public function init()
    {
		if (!self::$initialized) {
			parent::init();
			$this->components = $this->imanager->getCategory('name=Components');
			$this->csrf = Scriptor::getCSRF();
			$this->tplParser = new TemplateParser();
			include __DIR__.'/tpls/editor.php';
			self::$initialized = true;
		}
    }
    
    /**
     * Execute this module. 
     * 
     * This is the entry point for your module, it's triggered by the 
     * URL segment defined in your custom.scriptor-config.php file.
     * Example: 
     * 'modules' => [
	 *     'components' => [ <-- URL segment
     *         'menu' => 'Components',
     *         ...
     * 
     * @return void
     */
    public function execute() 
    {
        $this->checkAction();

        // Show component list
        if (!$this->segments->get(1)) { 
            $this->renderComponentList();
        } 
        // Edit component 
        elseif ($this->segments->get(1) == 'edit' && $this->input->get->comp) {
            $this->renderEdit();
        }
        // Create component
        elseif ($this->segments->get(1) == 'create' && !$this->input->get->comp) {
            $this->renderCreate();
        }
    }

    /**
     * Check user actions
     */
    public function checkAction()
    {
        // Save component
        if ($this->input->post->action == 'save-comp') {
            $this->saveComponent();
        }
        // Show component content in modal
		elseif ($this->input->post->action == 'render-markdown') {
			$this->renderContent();
		}
		// Delete component
		elseif ($this->segments->get(1) == 'delete' && $this->input->get->comp) {
			$this->removeComponent();
		}
		// Delete module
		elseif ($this->segments->get(1) == 'delete-components') {
			$this->uninstall();
		}
    }

    /**
     * Renders component list table.
     */
    protected function renderComponentList()
	{
        // Define page title
        $this->pageTitle = $this->i18n['cm_list_title'];

        // Breadcrumbs
        $this->breadcrumbs = $this->tplParser->render($this->tpls['bread_crumb_active'], [
            'label' => $this->i18n['cm_list_menu']
        ]);

		if ($this->components->items) {
			$rows = $this->renderRows();
        } else {
            $rows = $this->tplParser->render($this->tpls['empty_list_row'], [
                'content' => $this->i18n['cm_no_components']
            ]);
        }
        
        $this->pageContent = $this->tplParser->render($this->tpls['list_wrapper'], [
            'list_header' => $this->i18n['cm_list_header'],
            'id_table_header' => $this->i18n['cm_id_table_header'],
            'title_table_header' => $this->i18n['cm_title_table_header'],
            'options_table_header' => $this->i18n['cm_options_table_header'],
            'create_button' => $this->i18n['cm_create_button'],
            'rows' => $rows
        ]);

		//echo $this->pageContent;
	}

    /**
     * Renders the rows of the component list table
     */
    private function renderRows()
    {
        $rows = '';
		$sorted = $this->components->sort('position', 'asc',  0, 0, $this->components->items);
		$token = $this->csrf->renderUrl('&');
		foreach($sorted as $comp) {
            $rows .= $this->tplParser->render($this->tpls['comp_table_row'], [
                'comp_id' => $comp->id,
                'comp_title' => (mb_strlen($comp->name) > 80) ? mb_substr($comp->name, 0, 80).'...' : $comp->name,
                'pre_delete_msg' => $this->i18n['cm_pre_delete_msg'],
                'csrf_param' => $token
            ]);
		}
		return $rows;
    }

    /**
     * Renders the input form for new components
     */
    private function renderCreate()
    {
        // Define page title
        $this->pageTitle = $this->i18n['cm_list_title'];

        // Breadcrumbs
        $this->breadcrumbs = $this->tplParser->render($this->tpls['bread_crumb'], [
            'url' => '../',
            'label' => $this->i18n['cm_list_menu']
        ]);
        $this->breadcrumbs .= $this->tplParser->render($this->tpls['bread_crumb_active'], [
            'label' => $this->i18n['cm_create_menu']
        ]);
        // Build content
        $this->pageContent = $this->tplParser->render($this->tpls['create_form'], [
            'create_header' => $this->i18n['cm_create_header'],
            'form_url' => '',
            'title_label' => $this->i18n['cm_title_label'],
            'title_value' => $this->input->whitelist->name,
            'slug_label' => $this->i18n['cm_slug_label'],
            'slug_info' => $this->i18n['cm_slug_info'],
            'slug_value' => $this->input->whitelist->slug,
            'content_label' => $this->i18n['cm_content_label'],
            'content_value' => $this->input->whitelist->content,
            'create_button' => $this->i18n['cm_create_button'],
            'view_button' => $this->i18n['cm_view_button'],
            'csrf_inputs' => $this->csrf->renderInputs()
        ]);
    }

    /**
     * Renders the output mask for already existing components 
     */
    private function renderEdit()
    {
        $this->component = $this->components->getItem((int) $this->input->get->comp);
        if (!$this->component) {
            $this->msgs[] = array(
				'type' => 'error',
                'value' => $this->i18n['cm_comp_id_not_found']
            );
            Util::redirect("../");
        }
        // Define page title
        $this->pageTitle = $this->i18n['cm_list_title'];
        // Breadcrumbs
        $this->breadcrumbs = $this->tplParser->render($this->tpls['bread_crumb'], [
            'url' => '../',
            'label' => $this->i18n['cm_list_menu']
        ]);
        $this->breadcrumbs .= $this->tplParser->render($this->tpls['bread_crumb_active'], [
            'label' => $this->i18n['cm_edit_menu']
        ]);

        $this->pageContent = $this->tplParser->render($this->tpls['create_form'], [
            'create_header' => $this->i18n['cm_edit_header'],
            'form_url' => "?comp={$this->component->id}",
            'title_label' => $this->i18n['cm_title_label'],
            'title_value' => $this->component->name,
            'slug_label' => $this->i18n['cm_slug_label'],
            'slug_info' => $this->i18n['cm_slug_info'],
            'slug_value' => $this->component->slug,
            'content_label' => $this->i18n['cm_content_label'],
            'content_value' => $this->component->content,
            'create_button' => $this->i18n['cm_edit_button'],
            'view_button' => $this->i18n['cm_view_button'],
            'csrf_inputs' => $this->csrf->renderInputs()
        ]);
    }

    /**
     * Renders the content of the component for display in a modal. 
     * Sents a Json request and terminates the script execution. 
     * This method is called via Ajax.
     * 
     */
    private function renderContent() 
	{
		$parsedown = $this->loadModule('parsedown');
		$content = $this->tplParser->render($this->input->post->content, [
			'BASE_URL' => dirname($this->siteUrl).'/',
			'UPLOADS_URL' => dirname($this->siteUrl).'/data/uploads/'
		]);

		//if ($this->config['allowHtmlOutput'] === true) {
		$content = htmlspecialchars_decode($content);
        //}

        $return = $parsedown->text($content);

		Helper::sendJsonResponse(['status' => 1, 'text' => $return]);
    }
    
    /**
     * Deletes a component
     */
    private function removeComponent()
	{
		$component = $this->components->getItem((int) $this->input->get->comp);
		
		if ($this->config['protectCSRF'] && !$this->csrf->isTokenValid(
			$this->input->get->tokenName,
			$this->input->get->tokenValue, true)) {
				$this->msgs[] = array(
					'type' => 'error',
                    'value' => $this->i18n['error_csrf_token_mismatch']
                );
            Util::redirect('../');
			exit;
		}
		elseif ($component && $this->components->remove($component)) {
			$this->msgs[] = array(
				'type' => 'success',
                'value' => $this->i18n['cm_comp_successful_removed']
			);
			$this->imanager->sectionCache->expire();
			Util::redirect('../');
			exit;
		}

		$this->msgs[] = array(
			'type' => 'error',
            'value' => $this->i18n['cm_error_deleting_comp']
        );
        
		Util::redirect('../');
		exit;
	}

    /**
     * Checks user input and saves the component.
     * 
     * @return bool
     */
    private function saveComponent()
	{
		$this->component = null;

		if ($this->input->get->comp) { 
            $this->component = $this->components->getItem((int) $this->input->get->comp); 
        }
		if (!$this->component) { 
            $this->component = new Item($this->components->id); 
        }

		/* $this->msgs[] = [
			'type' => 'error',
			'value' => $this->i18n['cm_error_comp_content']
		]; */
		//Util::preformat($this->i18n['cm_error_comp_title']);
		//exit;
		//return false;

		$name = $this->imanager->sanitizer->text(str_replace('"', '', $this->input->post->name));
		if(!$name) {
			$this->msgs[] = array(
				'type' => 'error',
                'value' => $this->i18n['cm_error_comp_title']
			);
		}
		// Check if the name already exists
		$exists = $this->components->getItem("name=$name");
		if($exists && $exists->id != $this->component->id) {
			$this->msgs[] = array(
				'type' => 'error',
                'value' => $this->i18n['cm_error_comp_title_exists']
			);
		}
        $this->component->set('name', $name);
        $this->input->whitelist->name = $this->component->name;

		if ($this->input->post->slug) {
			$slug = preg_replace("/(-)\\1+/", "$1",
				$this->imanager->sanitizer->pageName($this->input->post->slug));
		} else {
            $url = trim($name, '-');
			$slug = preg_replace("/(-)\\1+/", "$1",
				$this->imanager->sanitizer->pageName($url));
		}
		
		// Invalid News name
		if (empty($this->msgs) && !$slug) {
			$this->msgs[] = array(
				'type' => 'error',
                'value' => $this->i18n['cm_error_comp_name']
			);
		}

        $this->component->set('slug', $slug);
        $this->input->whitelist->slug = $this->component->slug;

		$content = htmlentities($this->input->post->content);
		if (!$content) {
			$this->msgs[] = array(
				'type' => 'error',
                'value' => $this->i18n['cm_error_comp_content']
			);
		}
        $this->component->set('content', $content);
        $this->input->whitelist->content = $this->component->content;
		
		$this->component->active = true;

		if (!empty($this->msgs)) { return false; }

		if ($this->config['protectCSRF'] && !$this->csrf->isTokenValid(
			$this->input->post->tokenName,
			$this->input->post->tokenValue, true)) {
			$this->msgs[] = array(
				'type' => 'error',
                'value' => $this->i18n['error_csrf_token_mismatch']
			);
			return false;
		}

		if($this->component->save()) {
			$this->imanager->sectionCache->expire();
			$this->addMsg('success', $this->i18n['cm_successful_saved_comp']);
			/* $this->msgs[] = [
				'type' => 'success',
                'value' => $this->i18n['cm_successful_saved_comp']
			]; */
			Util::redirect("../edit/?comp={$this->component->id}");
		}
		return false;
	}

    /**
	 * Sends Json response and terminates the script execution.
	 *
	 * @param $data - Data to send
	 */
	private function sendJsonResponse($data)
	{
		header('Content-type: application/json; charset=utf-8');
		echo json_encode($data);
        exit;
    }
}

